@@ -65,7 +65,7 @@ class Client(Singleton):
 
     def get_qiniu_token(self, fname):
         qiniu_backend_keys = [k for k in list(self.backend_pools.keys()) if k.startswith('qiniu')]
-        if len(qiniu_backend_keys):
+        if len(qiniu_backend_keys) > 0:
             backend_key = random.sample(qiniu_backend_keys, 1)[0]
             backend = self.backend_pools[backend_key]
             return backend.get_token(fname)
@@ -73,6 +73,9 @@ class Client(Singleton):
     
     def update(self):
         with self.lock:
+            if self._inited:
+                return
+
             print('updating backends...')
             resp = httpx.get('https://{}/api/system/fs/backends'.format(self.host), headers={
                 'from': 'edge'
@@ -148,6 +151,21 @@ class Client(Singleton):
             return ret, e
         else:
             return True, cache_data
+
+    async def put_to_cloud_async(self, path, bucket, bucket_name, tmp=True):
+        cache_key = 'cloud_{}_{}'.format(bucket, bucket_name)
+        cache_data = self.get_cache(cache_key, toObj=False)
+        if cache_data is None:
+            prefix, backend = self._random_pick_backend()
+            
+            ret, e = await backend.put_async(path, bucket, bucket_name, tmp)
+            if ret:
+                url = '{}://{}'.format(prefix, e)
+                self.cache(cache_key, url)
+                return True, url
+            return ret, e
+        else:
+            return True, cache_data
     
     def get_from_cloud(self, path, url, force_copy=False, min_size=0):
         file_path = self.index_cache.get(url)
@@ -374,7 +392,7 @@ class Client(Singleton):
         
     
     async def put_async(self, path, bucket, bucket_name, timeout=10, file_ttl=-1):
-        ret, e = self.put_to_cloud(path, bucket, bucket_name)
+        ret, e = await self.put_to_cloud_async(path, bucket, bucket_name)
         if ret:
             ret, e = await self.put_from_cloud_async(e, bucket, bucket_name, timeout, file_ttl)
         return ret, e
